; Memorize Pi!
; Li Tao
; 9/13/01
; 1/15/01

#include "ti86asm.inc"

.org _asm_exec_ram

; Shell title
 nop
 jp Init
 .dw 0
 .dw VersionInfo

Init:
 ld hl,0
 ld (LastDigits),hl
; New guesses
Setup:
 call _runindicoff
 call _clrScrn
 call _homeup
 ld hl,VersionInfo
 call _puts
 call _newline
 ld hl,Prompt
 call _puts

; Loop for getting numbers
; bc = digit user is on
; hl = pointer to guess data
 ld bc,0
 ld hl,GuessData
GetNumberLoop:
 push hl
; Display cursor
 set curAble,(iy+curflags)
 call _getkey
 res curAble,(iy+curflags)
 pop hl
; Check keys
 cp kEE	       ; Cheat
 jp z,Cheat
 cp kF1	       ; Restore old correct value
 jp z,Restore
 cp kClear     ; Clear = Start over
 jp z,Setup
 cp kExit      ; Exit = Quit
 jp z,Quit
 cp kNext      ; More = Credits
 jp z,Info
 cp kDecPnt    ; Decimal point (treated like a digit)
 jp z,GetNumberLoopPoint

; No Del or Enter if there is no number yet
 push hl
 cp 0 ; clear c flag
 ld de,GuessData
 sbc hl,de
 pop hl
 jr z,SkipFullCommands
 cp kEnter     ; Enter = Check answer
 jp z,Enter
 cp kDel       ; Del = Backspace
 jr z,GetNumberLoopDel
 cp kLeft      ; Left arrow = Backspace
 jr z,GetNumberLoopDel
SkipFullCommands:
; Convert to binary, check for numbers
 sub k0
 cp 10
 jr nc,GetNumberLoop
; Convert to ASCII
 add a,48
 jr GetNumberLoopOK
Cheat:
 call Freebie
 jp GetNumberLoop

Freebie:
 push hl
 ld hl,DigitData
 add hl,bc
 ld a,(hl)
 pop hl
 ld (hl),a
 call _putc
 inc bc
 inc hl
 ret

Restore:
 push hl
 ld hl,(LastDigits)

 scf
 sbc hl,bc
 ld a,h
 cp 200
 pop hl
 jp nc,GetNumberLoop ; too high
 call Freebie
 jr Restore

; Delete up one column, adjusts screen
GetNumberLoopUpColumn:
 push de
 ld bc,(_curRow)
 dec bc
 ld (_curRow),bc
 pop de
 ld de,20
 jr GetNumberLoopDelBack
; Delete last digit
GetNumberLoopDel:
 push bc
; Clear spot at guess data
 ld a,Lspace
 ld (hl),a
 ld de,(_curCol)
 ld bc,(_curRow)

; Check for moving up a column
 ld a,e
 cp 0
 jr z,GetNumberLoopUpColumn
 dec de
GetNumberLoopDelBack:
; Clean up mess left by cursor and digit
 push de
 ld (_curCol),de
 ld a,(hl)
 call _putc
 call _putc
 pop de
 ld (_curRow),bc
 ld (_curCol),de
 pop bc

 dec bc
 dec hl
 jp GetNumberLoop
; Decimal point
GetNumberLoopPoint:
 ld a,Lperiod

; Digit key (0-9) or decimal point
GetNumberLoopOK:
 ld (hl),a
 call _putc
 inc bc
 inc hl
 jp GetNumberLoop

; Enter key
Enter:
 ld (hl),0
 ld (NumberDigits),bc
 call _homeup
 call _clrScrn

; Display your guess
 ld hl,YourGuess
 call _puts
 ld hl,GuessData
 ld de,DigitData
 ld c,0
; Checks for mistakes
MistakeLoop:
 ld a,(hl)
 ex de,hl
 ld b,(hl)
 ex de,hl
; Check for end of guess string
 cp 0
 jr z,DoneMistake
; Check for mistake
 cp b
 call nz,Mistake
 call _putc
 res 3,(iy+5)
 inc hl
 inc de
 jr MistakeLoop
DoneMistake:
 call _newline
 ld a,c
; Check if there was a mistake
 cp 1
 jr z,MessageWrong
; Right! Next:
 ld hl,Right
 call _puts
 ld hl,DigitData
 ld bc,(NumberDigits)
 add hl,bc
; Display next digits
 ld b,4
DoneMistakeLoop:
 ld a,(hl)
 call _putc
 inc hl
 djnz DoneMistakeLoop

; Display score
 call _newline
 ld hl,Score
 call _puts

 ld a,0

 ld hl,(NumberDigits)
 push hl
 ld (LastDigits),hl
; Display AHL
 call $4A33
 call _getkey
 pop hl

 jp Setup
MistakeMessageBack:
 call _getkey
 jp Setup
LowScore:
 ld hl,2
 ret
MessageWrong:
; Sorry, you're WRONG!
 ld hl,Wrong
 call _puts
 jr MistakeMessageBack

Quit:
 call _clrScrn
 call _homeup
 call _runindicon
 ret

Mistake:
; Set "wrong" register to 1
 ld c,1
; Invert mistakes
 set 3,(iy+5)
 ret

Info:
 call _clrScrn
; --- Memorize Pi! ---
 call _homeup
 set 3,(iy+5)
 ld hl,About
 call _puts
 res 3,(iy+5)
; Programmed by Li Tao
 call _newline
 ld hl,About2
 call _puts
; e-mail address
 ld bc,0
 ld (_penCol),bc
 ld bc,27
 ld (_penRow),bc
 ld hl,eMail
 call _vputs
 
; Version
 ld bc,46
 ld (_penCol),bc
 ld bc,57
 ld (_penRow),bc
 ld hl,Version
 call _vputs
 call _getkey
 jp Setup

NumberDigits:
 .db 0,0
DigitData:

 .db "3.141592653589793238462643383279502884197169399375105820974944592307816406286"
 .db "208998628034825342117067982148086513282306647093844609550582231725359408128"
 .db "481117450284102701938521105559644622948954930381964428810975665933446128475"
 .db "648233786783165271201909145648566923460348610454326648213393607260249141273"
 .db "724587006606315588174881520920962829254091715364367892590360011330530548820"
 .db "466521384146951941511609433057270365759591953092186117381932611793105118548"
 .db "074462379962749567351885752724891227938183011949129833673362440656643086021"
 .db "394946395224737190702179860943702770539217176293176752384674818467669405132"
 .db "000568127145263560827785771342757789609173637178721468440901224953430146549"
 .db "585371050792279689258923542019956112129021960864034418159813629774771309960"
 .db "518707211349999998372978049951059731732816096318595024459455346908302642522"
 .db "308253344685035261931188171010003137838752886587533208381420617177669147303"
 .db "598253490428755468731159562863882353787593751957781857780532171226806613001"
 .db "927876611195909216420198938095257201065485863278865936153381827968230301952"
 .db "035301852968995773622599413891249721775283479131515574857242454150695950829"
 .db "533116861727855889075098381754637464939319255060400927701671139009848824012"
 .db "858361603563707660104710181942955596198946767837449448255379774726847104047"
 .db "534646208046684259069491293313677028989152104752162056966024058038150193511"
 .db "253382430035587640247496473263914199272604269922796782354781636009341721641"
 .db "219924586315030286182974555706749838505494588586926995690927210797509302955"
 .db "321165344987202755960236480665499119881834797753566369807426542527862551818"
 .db "417574672890977772793800081647060016145249192173217214772350141441973568548"
 .db "161361157352552133475741849468438523323907394143334547762416862518983569485"
 .db "562099219222184272550254256887671790494601653466804988627232791786085784383"
 .db "827967976681454100953883786360950680064225125205117392984896084128488626945"
 .db "604241965285022210661186306744278622039194945047123713786960956364371917287"
 .db "46776465757396241389086583264599581339047802759010",0
About:
 .db " --- Memorize Pi --- ",0
About2:
 .db "Programmed by Li Tao",0
eMail:
 .db "e-mail: ltao@delta.fullerton.edu",0
Version:
 .db "Version 1.2",0
LastDigits:
 .dw 0
VersionInfo:
 .db "Memorize Pi! v1.2",0
YourGuess:
 .db "Your guess: ",0
Wrong:
 .db "Sorry, you're WRONG!",0
Right:
 .db "Right! Next: ",0
Score:
 .db "Score: ",0
Prompt:
 .db Lpi,"=",0

GuessData:
; Blank memory
 .db 0

.end
